from flask import Flask, render_template, abort, request
import math
# Модель данных
import books_model as model

app = Flask(__name__)

LINES_PER_PAGE = 15 # Число строк (точнее, абзацев) на одну страницу книги

# Представление, view
@app.route("/")
def main_page():
    books = model.books_list()
    return render_template("main.html", books=books)


@app.route("/read/<book_id>")
def read_page(book_id):
    try:
        book = model.find_book(book_id)
        text = model.get_book_text(book_id)
        # Вычисляем количество страниц книги
        pages = math.ceil(len(text) / LINES_PER_PAGE)
        try:
            # Получаем номер страницы из GET-параметров запроса
            page = int(request.args.get("page", "0"))
        except ValueError:
            # Если в параметре какая-то ерунда — считаем, что номер страницы 0
            page = 0
        if page < 0: # Если номер страницы отрицательный, зануляем его
            page = 0
        elif page >= pages: # Если номер страницы слишком большой, выбираем последнюю страницу
            page = pages - 1
        # Вычисляем индексы первого и последнего абзаца на странице
        start_index = page * LINES_PER_PAGE
        end_index = start_index + LINES_PER_PAGE
        # Вырезаем нужные абзацы из текста книги
        text = text[start_index:end_index]
        return render_template("read.html", book_id=book[2], title=book[1], lines=text, page=page, pages=pages)
    except KeyError:
        abort(404)


# Это просто пример работы с формой
@app.route("/form", methods=["GET", "POST"])
def form_page():
    errors = []
    if request.method == "POST": # Если форма была отправлена на обработку...
        # Вытаскиваем из формы значения полей, которые ввёл пользователь
        username = request.form["username"]
        email = request.form["email"]
        pass1 = request.form["password"]
        pass2 = request.form["password2"]
        is_agree = "agree" in request.form
        # Проверяем, что со значениями всё в порядке. Если нет, добавляем ошибки в список
        if pass1 != pass2:
            errors.append("Пароли не совпадают!")
        if not is_agree:
            errors.append("Нужно обязательно согласиться на условия обслуживания")
        
        if len(errors) == 0: # Если список ошибок таки пустой...
            # Тут как будто сохраняем данные...
            # И выводим страницу с сообщением, что всё у нас получилось:
            return render_template("success.html")
    # Иначе выводим страницу с формой
    return render_template("form.html", errors=errors)


@app.route("/search", methods=["GET", "POST"])
def search_page():
    show_res = False # Изначально не показываем списко результатов
    query_text = "" # Изначально текст для поиска пустой
    annot = False # Изначально галочка "Искать только по аннотациям" не стоит
    results = [] # Пустой список результатов поиска

    if request.method == "POST": # Если пользователь нажал кнопку "Искать", то есть отправил форму
        query_text = request.form["query"].strip() # Достаём из формы текст для поиска и убираем из него лишние пробелы
        if len(query_text) > 0: # Если текст есть, то есть поле не пустое
            annot = "annot" in request.form # Проверяем, стоит ли галочка "Искать только по аннотациям"
            show_res = True # Будем показывать результаты поиска
            results = model.search(query_text, annot, LINES_PER_PAGE) # Получаем список результатов поиска

    # Выводим страницу с формой (и результатами, если они есть)
    return render_template("search.html", 
        show_results=show_res, 
        query_text=query_text,
        annot=annot,
        results=results)